<?xml version="1.0" encoding="UTF-8"?>
<gui name="kreversi"
     version="3"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

<MenuBar>
  <Menu name="game">
    <Action name="game_seats" />
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="show_last_move" />
    <Action name="show_legal_moves" />
    <Action name="show_moves" />
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="anim_speed" />
    <Action name="skill" />
    <Action name="use_colored_chips" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="game_new" />
  <Separator />
  <Action name="move_undo" />
  <Action name="move_hint" />
  <Action name="move_demo" />
  <Separator />
  <Action name="show_last_move" />
  <Action name="show_legal_moves" />
</ToolBar>

</gui>
